package nl.uvt.locator;

import java.util.*;

import nu.xom.*;
import org.apache.log4j.*;

public class LinkInfo {
	String linkTitle;
	String linkIssn;
	String linkUrl;
	int linkVolume;
	int linkIssue;
	int rank;
	String linkMethod;
	String linkPublisher;
	String generatedUrl;
	List<Diagnostic> diagnostics;

	protected Logger logger;

	LinkInfo(){
		logger = Logger.getLogger(Locator.class);
		linkTitle = "";
		linkIssn = "";
		linkUrl = "";
		linkVolume = -1;
		linkIssue = -1;
		rank = 0;
		generatedUrl = "";
		linkPublisher="";
		linkMethod = "";
		diagnostics = new LinkedList<Diagnostic>();
	}

	protected String getValue( Node node, String val ){
		Nodes nodes = node.query( val );
		if ( nodes.size() > 0 ){
			return nodes.get(0).getValue();
		}
		else
			return "";
	}

	protected String[] getValues( Node node, String val ){
		Nodes nodes = node.query( val );
		if ( nodes.size() > 0 ){
			String[] result = new String[nodes.size()];
			for ( int i=0; i < nodes.size(); ++i )
				result[i] = nodes.get(i).getValue();
			return result;
		}
		else
			return null;
	}

	protected Node getXmlValue( Node node, String val ){
		Nodes nodes = node.query( val );
		if ( nodes.size() > 0 ){
			return nodes.get(0).copy();
		}
		else
			return null;
	}

	public boolean inRange( QueryData q ) throws LocatorException{
		return true;
	}

	public boolean applyMethod( QueryData pQ ){
		// TODO?
		return true;
	}


	Node toRecordNode(){
		Element record = new Element( "record" );
		record.appendChild( makeNode( "linkIssn", linkIssn ) );
		record.appendChild( makeNode( "linkTitle", linkTitle ) );
		record.appendChild( makeNode( "linkUrl", linkUrl) );
		record.appendChild( makeNode( "linkMethod", linkMethod ) );
//		record.appendChild( makeNode( "linkVolume", Integer.toString( linkVolume ) ) );
//		record.appendChild( makeNode( "linkIssue", Integer.toString( linkIssue ) ) );
		record.appendChild( makeNode( "rank", Integer.toString(rank) ) );
		record.appendChild( makeNode( "generatedUrl", generatedUrl) );
		record.appendChild( makeNode( "publisher", linkPublisher ) );
		if ( !diagnostics.isEmpty() )
			record.appendChild( diagnosticsToXML() );
		return record;
	}

	public static Element makeNode( String tag, String value ){
		Element node = new Element( tag );
		Text txt = new Text( value );
		node.appendChild( txt );
		return node;
	}

	public Element diagnosticsToXML() {
		Element diags = new Element( "diagnostics");
		for ( Iterator<Diagnostic> it=diagnostics.iterator(); it.hasNext(); ){
			Diagnostic dia = it.next();
			diags.appendChild( dia.toElement());
		}
		return diags;
	}
	
	public String generateOpenURL( QueryData pQ ){
		String openURL = "";
		if ( !pQ.genre.equals("") )
			openURL += "genre=" + pQ.genre;
		if ( !linkIssn.equals("") )
			openURL += "&issn=" + linkIssn;
		else if ( !pQ.issn.equals("") )
			openURL += "&issn=" + pQ.issn;
		else if ( !pQ.isbn.equals("") )
			openURL += "&isbn=" + pQ.isbn;
		if ( linkVolume > 0  )
			openURL += "&volume=" + linkVolume;
		else if ( pQ.volume > 0 )
			openURL += "&volume=" + pQ.volume;
		if ( linkIssue > 0  )
			openURL += "&issue=" + linkIssue;
		else if ( !pQ.issue.equals("") )
			openURL += "&issue=" + pQ.issue;
		if ( !pQ.spage.equals("") )
			openURL += "&spage=" + pQ.spage;
		if ( pQ.genre.equals("article")) {
			if ( !pQ.atitle.equals("") )
				openURL += "&atitle=" + pQ.atitle;
			if ( !pQ.title.equals("") )
			  openURL += "&title=" + pQ.title;
		}
		else if ( pQ.genre.equals("preprint")){
			if ( !pQ.atitle.equals("") )
				openURL += "&atitle=" + pQ.atitle;
		}
		else if ( pQ.genre.equals("proceeding")){
			if ( !pQ.atitle.equals("") )
				openURL += "&atitle=" + pQ.atitle;
		}
		else if ( pQ.genre.equals("book item")){
			if ( !pQ.atitle.equals("") )
				openURL += "&atitle=" + pQ.atitle;
		}
		else {
			if ( !pQ.atitle.equals("") )
				openURL += "&title=" + pQ.atitle;
			else if ( !pQ.title.equals("") )
				openURL += "&title=" + pQ.title;
		}

		if ( pQ.year > 0 )
			openURL += "&date=" + pQ.year;
		return openURL;
	}

}
